/**
 * AjaxCall class
 *
 * @module AjaxCall
 *
 * @author Luca Pau <luca.pau82@gmail.com>
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _log = require('../log');

var _log2 = _interopRequireDefault(_log);

var _result = Symbol();
var _message = Symbol();
var _value = Symbol();

var AjaxCall = (function () {
  function AjaxCall() {
    _classCallCheck(this, AjaxCall);

    this.result = 0;
    this.message = '';
    this.value = {};
  }

  _createClass(AjaxCall, [{
    key: 'sendErrorMessage',
    value: function sendErrorMessage(res, resultError, messageError) {
      this.result = resultError;
      this.message = messageError;
      res.status(resultError);
      this.send(res);
    }
  }, {
    key: 'sendUncaughtExceptionMessage',
    value: function sendUncaughtExceptionMessage(res, err) {
      _log2['default'].fatal('UncaughtException occurred on url:\n      ' + res.req.url + ' \n\n      with method "' + res.req.method + '"\n      This is the stack trace:\n\n      ' + (err.stack || 'no stack trace found') + '\n\n\n\n      Original error: \n\n      ' + err + '\n\n\n      Original error string formatted:\n      ' + this.constructor._stringifyError(err));
      this.sendErrorMessage(res, 500, 'UncaughtException');
    }
  }, {
    key: 'sendUnhandledExceptionMessage',
    value: function sendUnhandledExceptionMessage(res, err) {
      _log2['default'].error('UncaughtException occurred on url:\n      ' + res.req.url + ' \n\n      with method "' + res.req.method + '"\n      This is the stack trace:\n\n      ' + (err.stack || 'no stack trace found') + '\n\n\n\n      Original error: \n\n      ' + err + '\n\n\n      Original error string formatted:\n      ' + this.constructor._stringifyError(err));
      this.sendErrorMessage(res, 501, 'UnhandledException');
    }
  }, {
    key: 'send',
    value: function send(res) {
      return res.json(this.getStruct());
    }
  }, {
    key: 'sendValue',
    value: function sendValue(res, valueSent) {
      var statusCode = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];

      this.value = valueSent;
      res.status(statusCode || (this.constructor._isEmptyObj(this.value) ? 204 : 200));
      this.send(res);
    }
  }, {
    key: 'getStruct',
    value: function getStruct() {
      var success = this.result === 0;

      return {
        code: this.result,
        success: success,
        result: this.result,
        message: this.message,
        value: this.value
      };
    }
  }, {
    key: 'value',
    get: function get() {
      return this[_value];
    },
    set: function set(value) {
      this[_value] = value;
      return this;
    }
  }, {
    key: 'result',
    set: function set(resultValue) {
      this[_result] = resultValue;
      return this;
    },
    get: function get() {
      return this[_result];
    }
  }, {
    key: 'message',
    get: function get() {
      return this[_message];
    },
    set: function set(messageValue) {
      this[_message] = messageValue;
      return this;
    }
  }], [{
    key: '_isEmptyObj',
    value: function _isEmptyObj(obj) {
      return typeof obj === 'object' && Object.getOwnPropertyNames(obj).length === 0;
    }
  }, {
    key: '_stringifyError',
    value: function _stringifyError(err, filter) {
      var space = arguments.length <= 2 || arguments[2] === undefined ? '\n' : arguments[2];

      var plainObject = {};
      if (typeof err === 'object') {
        Object.getOwnPropertyNames(err).forEach(function (key) {
          plainObject[key] = err[key];
        });
      }
      return JSON.stringify(plainObject, filter, space);
    }
  }]);

  return AjaxCall;
})();

exports['default'] = AjaxCall;
module.exports = exports['default'];
